﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Messaging;
using System.Text;

/*
 *         Progetto: Esempio d'uso delle code MSMQ
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 18/01/2016
 *  
 *      Descrizione: Ricezione ed elaborazione di messaggi contenuti in una coda MSMQ.
 *                   
 *                   Aggiungere al progetto un riferimento all'assembly System.Messaging.
 *                   Per eseguire questa applicazione console, rendere pubblica la classe Ricevitore.
 */

namespace Ricevitore
{
    public class Ricevitore
    {
        static string nomeCoda = @".\Private$\Esempio";

        // Classe contenente i dettagli del messaggio ricevuto

        public class Info
        {
            public int[] Numeri { get; set; }

            public override string ToString()
            {
                if (Numeri == null || Numeri.Count() == 0)
                    return String.Empty;

                StringBuilder sb = new StringBuilder();
                int i = 1;

                sb.Append(Numeri[0]);

                while (i < Numeri.Count())
                    sb.Append(", " + Numeri[i++]);

                return sb.ToString();
            }
        }

        static void Main(string[] args)
        {
            Console.WriteLine(String.Format("Ricevitore avviato (coda {0})", nomeCoda));

            // Il metodo Exists può essere utilizzato solo per verificare l'esistenza
            // di code private locali (non funziona con le code private remote)

            if (!MessageQueue.Exists(nomeCoda))
            {

                // Se la coda non esiste occorre crearne una nuova.
                MessageQueue.Create(nomeCoda);

                // In alternativa è possibile creare una nuova coda dal prompt dei comandi
                // con il comando 
                // 
                // compmgmt.msc
                //
                // e selezionando Servizi e applicazioni -> Accodamento messaggi -> Code private
            }

            MessageQueue coda = new MessageQueue(nomeCoda);
            coda.Formatter = new XmlMessageFormatter(new Type[] { typeof(Info) });

            try {

                do {
                    Message m = coda.Receive();
                    Info inf = (Info) m.Body;

                    Elabora(inf);

                } while (true);

            }
            catch (MessageQueueException e) {
                Console.Error.WriteLine("Errore: {0}", e.Message);
            }
        }

        static void Elabora(Info inf) {
            int somma = 0;

            foreach(int i in inf.Numeri) {
                somma += i;
            }

            Console.WriteLine("La somma di {0} e' pari a {1}", inf, somma);
        }
    }
}
