﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Messaging;
using System.Text;

/*
 *         Progetto: Esempio d'uso delle code MSMQ
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 18/01/2016
 *  
 *      Descrizione: Trasmissione di un messaggio a una coda MSMQ di tipo Recoverable con
 *                   supporto dei messaggi di acknowledgment.
 *                   
 *                   Aggiungere al progetto un riferimento all'assembly System.Messaging.
 *                   Per eseguire questa applicazione console, rendere pubblica la classe TrasmettitoreRecAck.
 */

namespace TrasmettitoreRecAck
{
    public class TrasmettitoreRecAck
    {
        //public static string nomeCoda = @"FormatName:DIRECT=TCP:127.0.0.1\Private$\Esempio";
        public static string nomeCoda = @".\Private$\Esempio";
        public static string nomeCodaAck = @".\Private$\EsempioAck";

        public class Info
        {
            public int[] Numeri { get; set; }

            public override string ToString()
            {
                if (Numeri == null || Numeri.Count() == 0)
                    return String.Empty;

                StringBuilder sb = new StringBuilder();
                int i = 1;

                sb.Append(Numeri[0]);

                while (i < Numeri.Count())
                    sb.Append(", " + Numeri[i++]);

                return sb.ToString();
            }
        }

        static void Main(string[] args)
        {
            try
            {
                Console.WriteLine(String.Format("Trasmettitore avviato (coda {0})", nomeCoda));

                // Prima fase: Invio dei numeri da sommare

                MessageQueue coda = new MessageQueue(nomeCoda);

                Message m = new Message();
                m.Recoverable = true;
                m.AdministrationQueue = new MessageQueue(nomeCodaAck);
                m.AcknowledgeType = AcknowledgeTypes.FullReachQueue | AcknowledgeTypes.FullReceive;

                Random r = new Random();
                Info inf = new Info() { Numeri = new int[3] { r.Next(100), r.Next(100), r.Next(100) } };

                m.Body = inf;
                coda.Send(m);
                Console.WriteLine();
                Console.WriteLine("Inviata la sequenza {0}.", inf);
                Console.WriteLine();
                Console.WriteLine(String.Format("Id messaggio: {0}", m.Id));
                Console.WriteLine();

                // Seconda fase: elaborazione dei riscontri

                // Creare innanzitutto una coda privata sull'host del trasmettitore di nome EsempioAck con il comando compmgmt.msc

                MessageQueue codaAdmin = new MessageQueue(nomeCodaAck);
                Message mAdmin;
                codaAdmin.MessageReadPropertyFilter.CorrelationId = true;

                try
                {
                    // Si devono ricevere due riscontri: uno per la corretta consegna e uno per l'avvenuta elaborazione.
                    int numRiscontri = 2;

                    while (numRiscontri > 0)
                    {

                        // Rimane in attesa dei riscontri fino a 10 secondi
                        mAdmin = codaAdmin.Receive(TimeSpan.FromSeconds(10));

                        if (mAdmin.MessageType == MessageType.Acknowledgment)
                            switch (mAdmin.Acknowledgment)
                            {
                                case Acknowledgment.ReachQueue:
                                    Console.WriteLine("Messaggio recapitato con successo: {0}", mAdmin.CorrelationId);
                                    if (mAdmin.CorrelationId == m.Id)
                                        numRiscontri--;
                                    break;
                                case Acknowledgment.Receive:
                                    Console.WriteLine("Messaggio elaborato dal destinatario: {0}", mAdmin.CorrelationId);
                                    if (mAdmin.CorrelationId == m.Id)
                                        numRiscontri--;
                                    break;
                            }
                    }

                    Console.WriteLine();

                }
                catch (MessageQueueException e)
                {
                    Console.WriteLine(e.Message);
                }
            }
            catch (Exception e)
            {
                Console.Error.WriteLine("Errore: {0}", e.Message);
            }
        }
    }
}
